﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Printing;
using System.Windows.Documents;
using System.IO;
using System.Windows.Xps;
using System.Windows;

namespace ThinkShui.Libraries.External.Extensions
{
    public static class PrintRichTextBox
    {
        /// <summary>
        /// This version assumes that we will need to get the printer
        /// </summary>
        /// <param name="richTextBox"></param>
        public static void Print(this RichTextBox richTextBox)
        {
            PrintDialog pd = new PrintDialog();
            if ((pd.ShowDialog() == true))
            {
                PrintRichTextBox.Print(richTextBox, pd.PrintQueue, pd.PrintTicket);
            }
        }

        /// <summary>
        /// Actual method that does the printer
        /// Use this version if you already have the printer
        /// Code from http://blogs.msdn.com/b/prajakta/archive/2007/01/03/printing-contents-of-wpf-richtextbox.aspx
        /// </summary>
        /// <param name="richTextBox"></param>
        /// <param name="pq"></param>
        /// <param name="pt"></param>
        public static void Print(RichTextBox richTextBox, PrintQueue pq, PrintTicket pt = null)
        {
            TextRange sourceDocument = new TextRange(richTextBox.Document.ContentStart, richTextBox.Document.ContentEnd);
            MemoryStream stream = new MemoryStream();
            sourceDocument.Save(stream, DataFormats.Xaml);

            // Clone the source document's content into a new FlowDocument.
            FlowDocument flowDocumentCopy = new FlowDocument();
            TextRange copyDocumentRange = new TextRange(flowDocumentCopy.ContentStart, flowDocumentCopy.ContentEnd);
            copyDocumentRange.Load(stream, DataFormats.Xaml);

            // Create a XpsDocumentWriter object, open a Windows common print dialog.
            // This methods returns a ref parameter that represents information about the dimensions of the printer media. 
            XpsDocumentWriter docWriter = PrintQueue.CreateXpsDocumentWriter(pq);
            PageImageableArea ia = pq.GetPrintCapabilities().PageImageableArea;

            if (pt == null)
            {
                pt = pq.UserPrintTicket;
            }

            if (docWriter != null && ia != null)
            {
                DocumentPaginator paginator = ((IDocumentPaginatorSource)flowDocumentCopy).DocumentPaginator;
                // Change the PageSize and PagePadding for the document to match the CanvasSize for the printer device.
                paginator.PageSize = new Size((double)pt.PageMediaSize.Width, (double)pt.PageMediaSize.Height);
                Thickness pagePadding = flowDocumentCopy.PagePadding;
                flowDocumentCopy.PagePadding = new Thickness(
                        Math.Max(ia.OriginWidth, pagePadding.Left),
                        Math.Max(ia.OriginHeight, pagePadding.Top),
                        Math.Max((double)pt.PageMediaSize.Width - (double)(ia.OriginWidth + ia.ExtentWidth), pagePadding.Right),
                        Math.Max((double)pt.PageMediaSize.Height - (double)(ia.OriginHeight + ia.ExtentHeight), pagePadding.Bottom));
                flowDocumentCopy.ColumnWidth = double.PositiveInfinity;
                // Send DocumentPaginator to the printer.
                docWriter.Write(paginator);
            }
        }
    }
}
